<?php

namespace Drupal\inspect_test\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Drupal\Console\Core\Command\ContainerAwareCommand;
use Drupal\inspect\InspectHelperConfig;

use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Inspect test default command.
 *
 * CLI:
 * vendor/bin/drupal inspect_test:default
 *
 * Drupal\Console\Annotations\DrupalCommand (
 *     extension="inspect_test",
 *     extensionType="module"
 * )
 */
class DefaultCommand extends ContainerAwareCommand {

  /**
   * Super module (Inspect) configuration.
   *
   * @var InspectHelperConfig
   */
  protected $superModuleConfig;

  /**
   * @param ConfigFactoryInterface $config_factory
   */
  public function __construct(
    ConfigFactoryInterface $config_factory
  ) {
    $this->superModuleConfig = new InspectHelperConfig(
      $config_factory->get('inspect.settings')
    );

    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  protected function configure() {
    $this
      ->setName('inspect_test:default')
      ->setDescription('Logs a few inspections and traces.');
  }

  /**
   * CLI:
   * vendor/bin/drupal inspect_test:default
   *
   * {@inheritdoc}
   */
  protected function execute(InputInterface $input, OutputInterface $output) {
    /** @var \Drupal\inspect\Inspect $inspect */
    $inspect = \Drupal::service('inspect.inspect');

    // Variable inspection.
    /** @var \Drupal\inspect\Inspector $inspector */
    $inspector = $inspect->variable($this->superModuleConfig);
    $msg = 'inspect.settings:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->getIo()->info(
      "\n" . $msg . $inspector
    );

    $inspector = $inspect->variable($GLOBALS);
    $msg = '$GLOBALS:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->getIo()->info(
      "\n" . $msg . $inspector
    );

    $inspector = $inspect->variable($inspector);
    $msg = 'inspector self:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->getIo()->info(
      "\n" . $msg . $inspector
    );

    // Backtrace.
    $inspector = $inspect->trace(NULL);
    $msg = 'backtrace:';
    $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
    $this->getIo()->info(
      "\n" . $msg . $inspector
    );

    // Exception trace.
    try {
      throw new \RuntimeException('Doh!');
    }
    catch (\Throwable $xcptn) {
      $inspector = $inspect->trace($xcptn, ['limit' => 2]);
      $msg = 'exception:';
      $inspector->log('debug', $msg, ['channel' => 'inspect_test']);
      $this->getIo()->info(
        "\n" . $msg . $inspector
      );
    }

    // Shorthand functions.
    inspect($this->superModuleConfig, [
      'channel' => 'inspect_test',
      'message' => 'inspect(inspect.settings):',
    ]);
    inspect_trace(NULL, [
      'channel' => 'inspect_test',
      'message' => 'inspect_trace(NULL):',
    ]);
    // Shorthand non-array options.
    inspect($this->superModuleConfig, 2);
    inspect($this->superModuleConfig, (object) ['message' => 'inspect(..., object)']);
    inspect($this->superModuleConfig, 'inspect(..., message)');
    inspect(NULL, 'trace');
    inspect_trace(NULL, 3);

    return 0;
  }

}
