<?php
/**
 * @file
 * Contains \Drupal\inspect\Inspector.
 */

namespace Drupal\inspect;

/**
 * Produces variable inspection or backtrace,
 * stringable and loggable.
 */
class Inspector extends \SimpleComplex\Inspect\Inspector {

  /**
   * Only overridding for IDE variable resolution purposes.
   *
   * @var \Drupal\inspect\Inspect
   */
  protected $proxy;

  /**
   * Convenience method allowing method chaining,
   * e.g. \Drupal::service('inspect.inspect')->variable(...)->log().
   *
   * Works like and uses PSR logger->log().
   *
   * If $message is non-empty it gets prepended to inspection output.
   *
   * @see \Drupal::logger()
   * @see \Psr\Log\LoggerInterface::log()
   * @see \Psr\Log\LogLevel
   * @see \Drupal\Core\Logger\RfcLogLevel
   *
   * @param string|int $level
   * @param mixed $message
   *      Non-string becomes stringified.
   *      Non-empty gets prepended to inspection output.
   * @param mixed[] $context {
   *    @var string $channel  Optional, default 'inspect'.
   * }
   *
   * @return void
   */
  public function log($level = 'debug', $message = '', array $context = ['channel' => 'inspect']) {
    if (!empty($context['channel'])) {
      $channel = $context['channel'];
      unset($context['channel']);
    }
    else {
      $channel = 'inspect';
    }

    $this->proxy->getLoggerFactory()->get($channel)->log(
      $level,
      '' . $message . (!$message ? '' : "\n") . $this->__toString(),
      $context
    );
  }
}
